@@ -90,17 +90,18 @@ textmodel_lsa.dfm <- function(x, dim = 50L, min_count = 5L,
         rownames(wov) <- featnames(x)
     }
     result <- list(
-        values = wov,
-        dim = dim,
-        frequency = featfreq(x),
-        engine = engine,
-        weight = weight,
-        min_count = min_count,
-        concatenator = meta(x, field = "concatenator", type = "object"),
-        call = try(match.call(sys.function(-1), call = sys.call(-1)), silent = TRUE),
-        version = utils::packageVersion("wordvector")
+        "values" = list(word = wov),
+        "dim" = dim,
+        "frequency" = featfreq(x),
+        "engine" = engine,
+        "weight" = weight,
+        "min_count" = min_count,
+        "tolower" = tolower,
+        "concatenator" = meta(x, field = "concatenator", type = "object"),
+        "call" = try(match.call(sys.function(-1), call = sys.call(-1)), silent = TRUE),
+        "version" = utils::packageVersion("wordvector")
     )
-    class(result) <- "textmodel_wordvector"
+    class(result) <- c("textmodel_lsa", "textmodel_wordvector")
     return(result)
 }
 
